@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2007 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2009-2010 Steven Robertson
-#      2012,2013,2016 Nick Boultbee
+#           2012-2017 Nick Boultbee
 #           2009-2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from __future__ import absolute_import
 
@@ -43,6 +44,7 @@ from quodlibet.util.library import background_filter
 from quodlibet.util import connect_obj
 from quodlibet.qltk.cover import get_no_cover_pixbuf
 from quodlibet.qltk.image import add_border_widget, get_surface_for_pixbuf
+from quodlibet.qltk import popup_menu_at_widget
 
 
 class PreferencesButton(PreferencesButton):
@@ -120,6 +122,7 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
 
     _PATTERN_FN = os.path.join(quodlibet.get_user_dir(), "album_pattern")
     _DEFAULT_PATTERN_TEXT = DEFAULT_PATTERN_TEXT
+    STAR = ["~people", "album"]
 
     name = _("Cover Grid")
     accelerated_name = _("_Cover Grid")
@@ -153,12 +156,12 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
             covergrid.view.queue_resize()
 
     @classmethod
-    def toggle_vert(klass):
-        vert = config.getboolean("browsers", "covergrid_vertical", True)
+    def toggle_wide(klass):
+        wide = config.getboolean("browsers", "covergrid_wide", False)
         for covergrid in klass.instances():
             covergrid.songcontainer.set_orientation(
-                Gtk.Orientation.VERTICAL if vert
-                else Gtk.Orientation.HORIZONTAL)
+                Gtk.Orientation.HORIZONTAL if wide
+                else Gtk.Orientation.VERTICAL)
 
     @classmethod
     def update_mag(klass):
@@ -207,14 +210,14 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
         self.set_orientation(Gtk.Orientation.VERTICAL)
         self.songcontainer = qltk.paned.ConfigRVPaned(
             "browsers", "covergrid_pos", 0.4)
-        if not config.getboolean("browsers", "covergrid_vertical", True):
+        if config.getboolean("browsers", "covergrid_wide", False):
             self.songcontainer.set_orientation(Gtk.Orientation.HORIZONTAL)
 
         self._register_instance()
         if self.__model is None:
             self._init_model(library)
 
-        self._cover_cancel = Gio.Cancellable.new()
+        self._cover_cancel = Gio.Cancellable()
 
         self.scrollwin = sw = ScrolledWindow()
         sw.set_shadow_type(Gtk.ShadowType.IN)
@@ -307,11 +310,14 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
         view.drag_source_set(
             Gdk.ModifierType.BUTTON1_MASK, targets, Gdk.DragAction.COPY)
         view.connect("drag-data-get", self.__drag_data_get) # NOT WORKING
-        connect_obj(view, 'button-press-event', self.__popup, view, library)
+        connect_obj(view, 'button-press-event',
+            self.__rightclick, view, library)
+        connect_obj(view, 'popup-menu', self.__popup, view, library)
 
         self.accelerators = Gtk.AccelGroup()
         search = SearchBarBox(completion=AlbumTagCompletion(),
-                              accel_group=self.accelerators)
+                              accel_group=self.accelerators,
+                              star=self.STAR)
         search.connect('query-changed', self.__update_filter)
         connect_obj(search, 'focus-out', lambda w: w.grab_focus(), view)
         self.__search = search
@@ -374,7 +380,7 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
             if path is not None:
                 model.row_changed(path, model.get_iter(path))
             # XXX: icon view seems to ignore row_changed signals for pixbufs..
-            self.queue_resize()
+            self.queue_draw()
 
         item = model.get_value(iter_)
         scale_factor = self.get_scale_factor() * mag
@@ -396,8 +402,9 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
         model = self.view.get_model()
 
         self.__filter = None
-        if not Query.match_all(text):
-            self.__filter = Query(text, star=["~people", "album"]).search
+        query = self.__search.query
+        if not query.matches_all:
+            self.__filter = query.search
         self.__bg_filter = background_filter()
 
         self.__inhibit()
@@ -432,7 +439,7 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
         album = model.get_album(iter_)
         if album is None:
             return config.getboolean("browsers", "covergrid_all", False)
-        key = key.decode('utf-8').lower()
+        key = util.gdecode(key).lower()
         title = album.title.lower()
         if key in title:
             return False
@@ -443,7 +450,7 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
                     return False
         return True
 
-    def __popup(self, view, event, library):
+    def __rightclick(self, view, event, library):
         x = int(event.x)
         y = int(event.y)
         current_path = view.get_path_at_pos(x, y)
@@ -451,21 +458,26 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
             if not view.path_is_selected(current_path):
                 view.unselect_all()
             view.select_path(current_path)
-            albums = self.__get_selected_albums()
-            songs = self.__get_songs_from_albums(albums)
+            self.__popup(view, library)
 
-            items = []
-            num = len(albums)
-            button = MenuItem(
-                ngettext("Reload album _cover", "Reload album _covers", num),
-                Icons.VIEW_REFRESH)
-            button.connect('activate', self.__refresh_album, view)
-            items.append(button)
+    def __popup(self, view, library):
 
-            menu = SongsMenu(library, songs, items=[items])
-            menu.show_all()
-            menu.popup(None, None, None, event.button, event.time,
-                Gtk.get_current_event_time())
+        albums = self.__get_selected_albums()
+        songs = self.__get_songs_from_albums(albums)
+
+        items = []
+        num = len(albums)
+        button = MenuItem(
+            ngettext("Reload album _cover", "Reload album _covers", num),
+            Icons.VIEW_REFRESH)
+        button.connect('activate', self.__refresh_album, view)
+        items.append(button)
+
+        menu = SongsMenu(library, songs, items=[items])
+        menu.show_all()
+        popup_menu_at_widget(menu, view,
+            Gdk.BUTTON_SECONDARY,
+            Gtk.get_current_event_time())
 
     def _show_tooltip(self, widget, x, y, keyboard_tip, tooltip):
         w = self.scrollwin.get_hadjustment().get_value()
@@ -542,7 +554,7 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
 
     def filter_text(self, text):
         self.__search.set_text(text)
-        if Query.is_parsable(text):
+        if Query(text).is_parsable:
             self.__update_filter(self.__search, text)
             # self.__inhibit()
             #self.view.set_cursor((0,), None, False)
@@ -615,7 +627,7 @@ class CoverGrid(Browser, util.InstanceTracker, VisibleUpdate,
         entry.set_text(text)
 
         # update_filter expects a parsable query
-        if Query.is_parsable(text):
+        if Query(text).is_parsable:
             self.__update_filter(entry, text, scroll_up=False, restore=True)
 
         keys = config.gettext("browsers", "covergrid", "").split("\n")
