@@ -2,12 +2,13 @@
 # Copyright 2004-2008 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2009,2010 Steven Robertson
 #           2009-2013 Christoph Reiter
-#           2011,2013 Nick Boultbee
+#           2011-2017 Nick Boultbee
 #                2017 Fredrik Strupe
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk, GLib
 
@@ -152,13 +153,13 @@ class PanedBrowser(Browser, util.InstanceTracker):
         self._panes[-1].get_selection().emit('changed')
 
     def __added(self, library, songs):
-        songs = filter(self._filter, songs)
+        songs = list(filter(self._filter, songs))
         for pane in self._panes:
             pane.add(songs)
-            songs = filter(pane.matches, songs)
+            songs = list(filter(pane.matches, songs))
 
     def __removed(self, library, songs, remove_if_empty=True):
-        songs = filter(self._filter, songs)
+        songs = list(filter(self._filter, songs))
         for pane in self._panes:
             pane.remove(songs, remove_if_empty)
 
@@ -180,15 +181,16 @@ class PanedBrowser(Browser, util.InstanceTracker):
         return True
 
     def activate(self):
-        text = self._get_text()
-        if Query.is_parsable(text):
-            star = dict.fromkeys(SongList.star)
-            star.update(self.__star)
-            self._filter = Query(text, star.keys()).search
-            songs = filter(self._filter, self._library)
+        star = dict.fromkeys(SongList.star)
+        star.update(self.__star)
+        # TODO: get query from SearchBarBox (but with dynamic star)
+        query = Query(self._get_text(), star.keys())
+        if query.is_parsable:
+            self._filter = query.search
+            songs = list(filter(self._filter, self._library))
             bg = background_filter()
             if bg:
-                songs = filter(bg, songs)
+                songs = list(filter(bg, songs))
             self._panes[0].fill(songs)
 
     def scroll(self, song):
