@@ -4,8 +4,9 @@
 #           2013 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import sys
 
@@ -297,11 +298,7 @@ class SystemTray(BaseIndicator):
             self.__play_pause()
 
     def __play_pause(self, *args):
-        player = app.player
-        if player.song:
-            player.paused ^= True
-        else:
-            player.reset()
+        app.player.playpause()
 
     def __scroll(self, widget, event):
         state = event.get_state()
