@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
-#           2012-2016 Nick Boultbee
+#           2012-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 # Much of this code is highly optimized, because many of the functions
 # are called in tight loops. Don't change things just to make them
@@ -16,7 +17,7 @@ import time
 
 from senf import fsn2uri, fsnative, fsn2text, devnull, bytes2fsn, path2fsn
 
-from quodlibet import _
+from quodlibet import _, print_d
 from quodlibet import util
 from quodlibet import config
 from quodlibet.util.path import mkdir, mtime, expanduser, \
@@ -76,12 +77,13 @@ def decode_value(tag, value):
     Not reversible.
     """
 
-    if isinstance(value, text_type):
-        return value
-    elif isinstance(value, float):
-        return u"%.2f" % value
-    elif tag in FILESYSTEM_TAGS:
+    if tag in FILESYSTEM_TAGS:
         return fsn2text(value)
+    elif tag[:2] == "~#":
+        if isinstance(value, float):
+            return u"%.2f" % value
+        else:
+            return text_type(value)
     return text_type(value)
 
 
@@ -126,6 +128,12 @@ class AudioFile(dict, ImageContainer):
     mimes = []
     """MIME types this class can represent"""
 
+    def __init__(self, default=tuple(), **kwargs):
+        for key, value in dict(default).items():
+            self[key] = value
+        for key, value in kwargs.items():
+            self[key] = value
+
     def __song_key(self):
         return (self("~#disc", 1), self("~#track", 1),
             human(self("artistsort")),
@@ -286,17 +294,23 @@ class AudioFile(dict, ImageContainer):
     def iterrealitems(self):
         return ((k, v) for (k, v) in iteritems(self) if k[:1] != "~")
 
-    def __call__(self, key, default=u"", connector=" - "):
-        """Return a key, synthesizing it if necessary. A default value
-        may be given (like dict.get); the default default is an empty
-        unicode string (even if the tag is numeric).
+    def __call__(self, key, default=u"", connector=" - ", joiner=', '):
+        """Return the value(s) for a key, synthesizing if necessary.
+        Multiple values for a key are delimited by newlines.
 
-        If a tied tag ('a~b') is requested, the 'connector' keyword
+        A default value may be given (like `dict.get`);
+        the default default is an empty unicode string
+        (even if the tag is numeric).
+
+        If a tied tag ('a~b') is requested, the `connector` keyword
         argument may be used to specify what it is tied with.
         In case the tied tag contains numeric and file path tags, the result
         will still be a unicode string.
+        The `joiner` keyword specifies how multiple *values* will be joined
+        within that tied tag output, e.g.
+            ~people~title = "Kanye West, Jay Z - New Day"
 
-        For details on tied tags, see the documentation for util.tagsplit.
+        For details on tied tags, see the documentation for `util.tagsplit`.
         """
 
         if key[:1] == "~":
@@ -304,8 +318,13 @@ class AudioFile(dict, ImageContainer):
             if "~" in key:
                 real_key = "~" + key
                 values = []
-                for v in map(self.__call__, util.tagsplit(real_key)):
-                    v = decode_value(real_key, v)
+                sub_tags = util.tagsplit(real_key)
+                # If it's genuinely a tied tag (not ~~people etc), we want
+                # to delimit the multi-values separately from the tying
+                j = joiner if len(sub_tags) > 1 else "\n"
+                for t in sub_tags:
+                    vs = [decode_value(real_key, v) for v in (self.list(t))]
+                    v = j.join(vs)
                     if v:
                         values.append(v)
                 return connector.join(values) or default
@@ -428,11 +447,13 @@ class AudioFile(dict, ImageContainer):
 
                 # If there are no embedded lyrics, try to read them from
                 # the external file.
+                fn = self.lyric_filename
                 try:
-                    fileobj = open(self.lyric_filename, "rU")
+                    fileobj = open(fn, "rUb")
                 except EnvironmentError:
                     return default
                 else:
+                    print_d("Reading lyrics from %s" % fn)
                     return fileobj.read().decode("utf-8", "replace")
             elif key == "filesize":
                 return util.format_size(self("~#filesize", 0))
@@ -572,7 +593,7 @@ class AudioFile(dict, ImageContainer):
 
     def list(self, key):
         """Get all values of a tag, as a list. Synthetic tags are supported,
-        but will be slower. Numeric tags are not supported.
+        but will be slower. Numeric tags will give their one value.
 
         For file path keys the returned list might contain path items
         (non-unicode).
@@ -582,11 +603,11 @@ class AudioFile(dict, ImageContainer):
         """
 
         if "~" in key or key == "title":
-            v = self(key, connector="\n")
+            v = self(key)
             if v == "":
                 return []
             else:
-                return v.split("\n")
+                return v.split("\n") if isinstance(v, text_type) else [v]
         else:
             v = self.get(key)
             return [] if v is None else v.split("\n")
@@ -734,6 +755,7 @@ class AudioFile(dict, ImageContainer):
 
         # Replace nulls with newlines, trimming zero-length segments
         for key, val in listitems(self):
+            self[key] = val
             if isinstance(val, string_types) and '\0' in val:
                 self[key] = '\n'.join(filter(lambda s: s, val.split('\0')))
             # Remove unnecessary defaults
