@@ -2,51 +2,32 @@
 # Copyright 2013-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
-
-from gi.repository import Gtk
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from quodlibet import ngettext, _
 from quodlibet.qltk import get_top_parent, get_menu_item_top_parent
-from quodlibet.qltk.msg import WarningMessage
+from quodlibet.qltk.msg import ConfirmationPrompt
 from quodlibet.qltk.x import SeparatorMenuItem
-from quodlibet.qltk import Icons
 from quodlibet.util import print_exc, format_int_locale
 from quodlibet.util.dprint import print_d, print_e
 from quodlibet.plugins import PluginHandler, PluginManager
 from quodlibet.plugins.gui import MenuItemPlugin
 
 
-class ConfirmMultiPlaylistInvoke(WarningMessage):
+def confirm_multi_playlist_invoke(parent, plugin_name, count):
     """Dialog to confirm invoking a plugin with X playlists
     in case X is high
     """
-
-    RESPONSE_INVOKE = 1
-
-    def __init__(self, parent, plugin_name, count):
-        params = {"name": plugin_name, "count": format_int_locale(count)}
-        title = ngettext("Run the plugin \"%(name)s\" on %(count)s playlist?",
-                         "Run the plugin \"%(name)s\" on %(count)s playlists?",
-                         count) % params
-
-        super(ConfirmMultiPlaylistInvoke, self).__init__(
-            get_top_parent(parent),
-            title, "",
-            buttons=Gtk.ButtonsType.NONE)
-
-        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
-        self.add_icon_button(
-            _("_Run Plugin"), Icons.SYSTEM_RUN, self.RESPONSE_INVOKE)
-        self.set_default_response(Gtk.ResponseType.CANCEL)
-
-    @classmethod
-    def confirm(cls, parent, plugin_name, count):
-        """Returns if the action was confirmed"""
-
-        resp = cls(parent, plugin_name, count).run()
-        return resp == cls.RESPONSE_INVOKE
+    params = {"name": plugin_name, "count": format_int_locale(count)}
+    title = ngettext("Run the plugin \"%(name)s\" on %(count)s playlist?",
+                     "Run the plugin \"%(name)s\" on %(count)s playlists?",
+                     count) % params
+    description = ""
+    ok_text = _("_Run Plugin")
+    prompt = ConfirmationPrompt(parent, title, description, ok_text).run()
+    return prompt == ConfirmationPrompt.RESPONSE_INVOKE
 
 
 class PlaylistPlugin(MenuItemPlugin):
@@ -108,7 +89,7 @@ class PlaylistPluginHandler(PluginHandler):
 
         self.__plugins = []
         self._confirm_multiple = (confirmer or
-                                  ConfirmMultiPlaylistInvoke.confirm)
+                                  confirm_multi_playlist_invoke)
 
     def populate_menu(self, menu, library, browser, playlists):
         """Appends items onto `menu` for each enabled playlist plugin,
