@@ -3,13 +3,15 @@
 #                     Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 
 from gi.repository import Gtk, Pango, GObject
 
+from quodlibet import _
 from quodlibet import qltk
 from quodlibet.qltk.views import RCMHintedTreeView
 from quodlibet.util import connect_obj
@@ -193,7 +195,7 @@ class StandaloneEditor(_KeyValueEditor):
         if os.path.exists(filename):
             fileobj = open(filename, "rU")
             lines = list(fileobj.readlines())
-            for i in range(len(lines) / 2):
+            for i in range(len(lines) // 2):
                 ret.append((lines[i * 2 + 1].strip(), lines[i * 2].strip()))
         return ret
 
@@ -213,7 +215,6 @@ class StandaloneEditor(_KeyValueEditor):
                 self.model.prepend(
                     row=[lines.pop(1).strip(), lines.pop(0).strip()])
         if not len(self.model) and self.initial:
-            #print_d("None found - using defaults.", context=self)
             for (k, v) in self.initial:
                 self.model.append(row=[v.strip(), k.strip()])
 
