@@ -1,17 +1,18 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
 #           2016 Ryan Dellenbaugh
+#           2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import codecs
 import re
 
 from . import _match as match
 from ._match import ParseError
-from ._diacritic import re_add_variants
 from quodlibet.util import re_escape
 from quodlibet.compat import text_type, PY3
 
@@ -116,12 +117,16 @@ class QueryParser(object):
         elif self.accept('@'):
             return self.Extension()
         try:
-            # Equals and Star can begin the same, so try Equals first then
-            # backtrack and try Star on failure
+            # Equals, NotEquals and Star can begin the same,
+            # so try in order, backtracking on failure (with Star last)
             index = self.index
             return self.Equals()
         except ParseError:
             self.index = index
+            try:
+                return self.NotEquals()
+            except ParseError:
+                self.index = index
             return self.Star(outer=outer)
 
     def Negation(self, rule):
@@ -240,6 +245,15 @@ class QueryParser(object):
         value = self.Value()
         return match.Tag(tags, value)
 
+    def NotEquals(self):
+        """Rule for 'tag!=value' queries"""
+        tags = self.match_list(lambda: self.expect_re(TAG))
+        tags = [tag.strip() for tag in tags]
+        self.expect('!')
+        self.expect('=')
+        value = self.Value()
+        return match.Neg(match.Tag(tags, value))
+
     def Value(self, outer=False):
         """Rule for value. Either a regexp, quoted string, boolean combination
         of values, or free text string"""
@@ -266,35 +280,15 @@ class QueryParser(object):
                 # Hack to force plain text parsing for top level free text
                 raise ParseError('Free text not allowed at top level of query')
 
-            return self.RegexpMods(re_escape(self.expect_re(TEXT)), False)
+            return match.Regex(re_escape(self.expect_re(TEXT)), u"d")
 
-    def RegexpMods(self, regex, mods=True):
+    def RegexpMods(self, regex):
         """Consume regexp modifiers from tokens and compile provided regexp
-        with them."""
-        if mods:
-            mod_string = self.expect_re(MODIFIERS)
-        else:
-            mod_string = u""
-        mods = re.MULTILINE | re.UNICODE | re.IGNORECASE
-        if "c" in mod_string:
-            mods &= ~re.IGNORECASE
-        if "i" in mod_string:
-            mods |= re.IGNORECASE
-        if "s" in mod_string:
-            mods |= re.DOTALL
-        if "l" in mod_string:
-            mods = (mods & ~re.UNICODE) | re.LOCALE
-        if "d" in mod_string:
-            try:
-                regex = re_add_variants(regex)
-            except re.error:
-                raise ParseError("The regular expression was invalid")
-            except NotImplementedError:
-                raise ParseError("The regular expression was is not supported")
-        try:
-            return re.compile(regex, mods)
-        except re.error:
-            raise ParseError("The regular expression /%s/ is invalid." % regex)
+        with them.
+        """
+
+        mod_string = self.expect_re(MODIFIERS)
+        return match.Regex(regex, mod_string)
 
     def Star(self, outer=False):
         """Rule for value that matches all visible tags"""
